#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Do something for uninstalling the TMANS."""

import ionas.request as request
from fal_info import (FAL_SERVICECTRL_PATH, FAL_CLONE_PATH, ReqTypes)

import manualctrl


CLONE_WATCHDOG = "clone_watchdog"
SERVICELIST_NAME = "servicelist"
SERVICE_NAME = "service"
NAME_NAME = "name"
SERVICECTRL_CLONE_WATCHDOG_REQ_XML = """<servicectrl>
    <servicelist>
        <service><name>%s</name></service>
    </servicelist>
</servicectrl>
""" % (CLONE_WATCHDOG, )

CLONE_QUERY_XML = "<clone/>"
CLONE_DISABLE_MASTER_XML = """<clone>
    <master enable="false">
        <slave><host/><passwd/></slave>
    </master>
</clone>
"""

MASTER_NAME = "master"


def __send_request(target, type, body=""):
    """Send request entry to "target"."""

    entry = request.Entry(type, body)
    return request.Request(target).request(entry).get_entry()


def __is_master_enabled():
    """Check if the master function is enabled."""

    try:
        query_result = __send_request(FAL_CLONE_PATH, ReqTypes.QUERY,
                                        CLONE_QUERY_XML)
        if query_result.get().get_node(MASTER_NAME).is_enabled():
            return True
    except:
        pass

    return False


def try_stop_hclone_service():
    """Try_stop service for the clone package."""

    try:
        __send_request(FAL_SERVICECTRL_PATH, ReqTypes.STOP,
                            SERVICECTRL_CLONE_WATCHDOG_REQ_XML)
    except:
        # Ignore error
        pass


def delete_hclone_service():
    """Add service for the clone package to a servicectrl's configuration
       file.
    """

    try:
        servicectrl_config = \
                    __send_request(FAL_SERVICECTRL_PATH, ReqTypes.LOAD).get()

        servicelist = servicectrl_config.get_node(SERVICELIST_NAME)
        for (idx, service) in \
                enumerate(servicelist.get_node_list(SERVICE_NAME)):

            try:
                if service.get_node(NAME_NAME).get_text() == CLONE_WATCHDOG:
                    servicelist.delete_child(SERVICE_NAME, index=idx)
                    __send_request(FAL_SERVICECTRL_PATH, ReqTypes.SAVE,
                                    servicectrl_config)
                    return
            except:
                pass
    except:
        pass


def try_disable_master():
    """Try disable master function."""

    try:
        if __is_master_enabled():
            __send_request(FAL_CLONE_PATH, ReqTypes.APPLY,
                            CLONE_DISABLE_MASTER_XML)
    except:
        # Ignore error
        pass


#
# Main
#

try_stop_hclone_service()
delete_hclone_service()
try_disable_master()

manualctrl.uninstall()
